@@ -18,16 +18,71 @@
 
 #define G_LOG_DOMAIN "GcalEvent"
 
+#include "gconstructor.h"
 #include "gcal-event.h"
 #include "gcal-utils.h"
+#include "gcal-recurrence.h"
 
 #define LIBICAL_TZID_PREFIX "/freeassociation.sourceforge.net/"
 
+/**
+ * SECTION:gcal-event
+ * @short_description: A class that represents an event
+ * @title:GcalEvent
+ * @stability:unstable
+ * @see_also:GcalEventWidget,GcalManager
+ *
+ * The #GcalEvent class represents an appointment, with
+ * various functions to modify it. All the changes are
+ * transient. To persistently store the changes, you
+ * need to call gcal_manager_update_event().
+ *
+ * Although the #ECalComponent may have no end date. In this
+ * case, gcal_event_get_date_end() returns the same date that
+ * gcal_event_get_date_start().
+ *
+ * #GcalEvent implements #GInitable, and creating it possibly
+ * can generate an error. At the moment, the only error that
+ * can be generate is #GCAL_EVENT_ERROR_INVALID_START_DATE.
+ *
+ * ## Timezones
+ *
+ * When a #GcalEvent is created, the timezone is parsed from
+ * the #ECalComponent. The start and end dates can possibly
+ * have different timezones, e.g. when the user is traveling
+ * across timezones and the departure time is in a different
+ * timezone of the arrival time.
+ *
+ * For the sake of sanity, gcal_event_get_timezone() returns
+ * the timezone of the start date. If you need to precisely
+ * check the timezones of the start and end dates, you have
+ * to use g_date_time_get_timezone_identifier().
+ *
+ * ## Example:
+ * |[<!-- language="C" -->
+ * GcalEvent *event;
+ * GError *error;
+ *
+ * error = NULL;
+ * event = gcal_event_new (source, component, &error);
+ *
+ * if (error)
+ *   {
+ *     g_warning ("Error creating event: %s", error->message);
+ *     g_clear_error (&error);
+ *     return;
+ *   }
+ *
+ * ...
+ * ]|
+ */
+
 struct _GcalEvent
 {
   GObject             parent;
 
   gchar              *uid;
+  gboolean            has_recurrence;
 
   /*
    * The description is cached in the class because it
@@ -51,6 +106,8 @@ struct _GcalEvent
   ECalComponent      *component;
   ESource            *source;
 
+  GcalRecurrence     *recurrence;
+
   gboolean            is_valid : 1;
   GError             *initialization_error;
 };
@@ -73,9 +130,44 @@ enum {
   PROP_SUMMARY,
   PROP_TIMEZONE,
   PROP_UID,
+  PROP_HAS_RECURRENCE,
+  PROP_RECURRENCE,
   N_PROPS
 };
 
+
+/*
+ * GcalEvent cache
+ */
+
+static GHashTable *event_cache = NULL;
+
+G_DEFINE_CONSTRUCTOR (init_event_cache_map);
+
+static void
+init_event_cache_map (void)
+{
+  event_cache = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
+}
+
+G_DEFINE_DESTRUCTOR (destroy_event_cache_map);
+
+static void
+destroy_event_cache_map (void)
+{
+  GList *events;
+
+  /* Destroy all events */
+  events = g_hash_table_get_values (event_cache);
+  g_list_free_full (events, g_object_unref);
+
+  g_hash_table_destroy (event_cache);
+}
+
+/*
+ * Auxiliary methods
+ */
+
 static GTimeZone*
 get_timezone_from_ical (ECalComponentDateTime *comp)
 {
@@ -129,9 +221,8 @@ get_timezone_from_ical (ECalComponentDateTime *comp)
 }
 
 static ECalComponentDateTime*
-build_component_from_datetime (GcalEvent    *self,
-                               icaltimezone *current_tz,
-                               GDateTime    *dt)
+build_component_from_datetime (GcalEvent *self,
+                               GDateTime *dt)
 {
   ECalComponentDateTime *comp_dt;
 
@@ -139,15 +230,19 @@ build_component_from_datetime (GcalEvent    *self,
     return NULL;
 
   comp_dt = g_new0 (ECalComponentDateTime, 1);
-  comp_dt->value = NULL;
-  comp_dt->tzid = NULL;
-
   comp_dt->value = datetime_to_icaltime (dt);
-  comp_dt->value->zone = self->all_day ? icaltimezone_get_utc_timezone () : current_tz;
   comp_dt->value->is_date = self->all_day;
 
-  /* All day events have UTC timezone */
-  comp_dt->tzid = g_strdup (self->all_day ? "UTC" : icaltimezone_get_tzid (current_tz));
+  if (self->all_day)
+    {
+      comp_dt->value->zone = icaltimezone_get_utc_timezone ();
+      comp_dt->tzid = g_strdup ("UTC");
+    }
+  else
+    {
+      comp_dt->value->zone = e_cal_util_get_system_timezone ();
+      comp_dt->tzid = g_strdup (icaltimezone_get_tzid ((icaltimezone *) comp_dt->value->zone));
+    }
 
   return comp_dt;
 }
@@ -311,9 +406,17 @@ gcal_event_set_component_internal (GcalEvent     *self,
       /* Setup UID */
       gcal_event_update_uid_internal (self);
 
+      /* Set has-recurrence to check if the component has recurrence or not */
+      self->has_recurrence = e_cal_component_has_recurrences(component);
+
+      /* Load the recurrence-rules in GcalRecurrence struct */
+      self->recurrence = gcal_recurrence_parse_recurrence_rules (component);
+
       /* Load and setup the alarms */
       load_alarms (self);
 
+      g_object_notify (G_OBJECT (self), "has-recurrence");
+      g_object_notify (G_OBJECT (self), "recurrence");
       g_object_notify (G_OBJECT (self), "component");
       g_object_notify (G_OBJECT (self), "location");
       g_object_notify (G_OBJECT (self), "summary");
@@ -362,6 +465,10 @@ gcal_event_finalize (GObject *object)
 {
   GcalEvent *self = (GcalEvent *)object;
 
+  g_debug ("Removing '%s' (%p) from cache", self->uid, self);
+
+  g_hash_table_remove (event_cache, self->uid);
+
   g_clear_pointer (&self->dt_start, g_date_time_unref);
   g_clear_pointer (&self->dt_end, g_date_time_unref);
   g_clear_pointer (&self->timezone, g_time_zone_unref);
@@ -371,6 +478,7 @@ gcal_event_finalize (GObject *object)
   g_clear_pointer (&self->color, gdk_rgba_free);
   g_clear_object (&self->component);
   g_clear_object (&self->source);
+  g_clear_pointer (&self->recurrence, gcal_recurrence_unref);
 
   G_OBJECT_CLASS (gcal_event_parent_class)->finalize (object);
 }
@@ -429,6 +537,14 @@ gcal_event_get_property (GObject    *object,
       g_value_set_string (value, self->uid);
       break;
 
+    case PROP_HAS_RECURRENCE:
+      g_value_set_boolean (value, self->has_recurrence);
+      break;
+
+    case PROP_RECURRENCE:
+      g_value_set_boxed (value, self->recurrence);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -484,6 +600,10 @@ gcal_event_set_property (GObject      *object,
       gcal_event_set_timezone (self, g_value_get_boxed (value));
       break;
 
+    case PROP_RECURRENCE:
+      gcal_event_set_recurrence (self, g_value_get_boxed (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -640,6 +760,33 @@ gcal_event_class_init (GcalEventClass *klass)
                                                         "The unique identifier of the event",
                                                         "",
                                                         G_PARAM_READABLE));
+
+  /**
+  * GcalEvent::has-recurrence:
+  *
+  * The recurrence property of the event.
+  */
+  g_object_class_install_property (object_class,
+                                   PROP_HAS_RECURRENCE,
+                                   g_param_spec_boolean ("has-recurrence",
+                                                         "If event has recurrence",
+                                                         "Whether the event has recurrence or not",
+                                                         FALSE,
+                                                         G_PARAM_READABLE));
+
+  /**
+  * GcalEvent::recurrence:
+  *
+  * The recurrence-rules property of the event.
+  */
+  g_object_class_install_property (object_class,
+                                   PROP_RECURRENCE,
+                                   g_param_spec_boxed ("recurrence",
+                                                       "Recurrence property of the event",
+                                                       "The recurrence property of the event",
+                                                       GCAL_TYPE_RECURRENCE,
+                                                       G_PARAM_READWRITE));
+
 }
 
 static void
@@ -685,12 +832,36 @@ gcal_event_new (ESource        *source,
                 ECalComponent  *component,
                 GError        **error)
 {
-  return g_initable_new (GCAL_TYPE_EVENT,
-                         NULL,
-                         error,
-                         "source", source,
-                         "component", component,
-                         NULL);
+  GcalEvent *event;
+  g_autofree gchar *uuid;
+
+  uuid = get_uuid_from_component (source, component);
+
+  if (g_hash_table_contains (event_cache, uuid))
+    {
+      g_debug ("Using cached value for %s", uuid);
+
+      event = g_hash_table_lookup (event_cache, uuid);
+      gcal_event_set_component_internal (event, component);
+      g_object_ref (event);
+    }
+  else
+    {
+      event = g_initable_new (GCAL_TYPE_EVENT,
+                              NULL,
+                              error,
+                              "source", source,
+                              "component", component,
+                              NULL);
+
+      if (event)
+        {
+          g_debug ("Adding %s to the cache", event->uid);
+          g_hash_table_insert (event_cache, event->uid, event);
+        }
+    }
+
+  return event;
 }
 
 /**
@@ -731,7 +902,7 @@ gcal_event_get_color (GcalEvent *self)
  * @color: a #GdkRGBA
  *
  * Sets the color of the event, which is passed to the
- * parent @ESource.
+ * parent #ESource.
  */
 void
 gcal_event_set_color (GcalEvent *self,
@@ -789,7 +960,9 @@ gcal_event_set_all_day (GcalEvent *self,
  * gcal_event_get_date_end:
  * @self: a #GcalEvent
  *
- * Retrieves the end date of @self.
+ * Retrieves the end date of @self. If the component doesn't
+ * have an end date, then the returned date is the same of
+ * gcal_event_get_date_start().
  *
  * Returns: (transfer none): a #GDateTime.
  */
@@ -815,29 +988,21 @@ gcal_event_set_date_end (GcalEvent *self,
 {
   g_return_if_fail (GCAL_IS_EVENT (self));
 
-  if (self->dt_end != dt &&
-      (!self->dt_end || !dt ||
-       (self->dt_end && dt && !g_date_time_equal (self->dt_end, dt))))
+  if (self->dt_end != dt)
     {
-      ECalComponentDateTime *component_dt, current;
+      ECalComponentDateTime *component_dt;
 
       g_clear_pointer (&self->dt_end, g_date_time_unref);
       self->dt_end = g_date_time_ref (dt);
 
-      /* Retrieve the current timezone */
-      e_cal_component_get_dtstart (self->component, &current);
-
       /* Setup the ECalComponent's datetime value */
-      component_dt = build_component_from_datetime (self,
-                                                    icaltimezone_get_builtin_timezone_from_tzid (current.tzid),
-                                                    dt);
+      component_dt = build_component_from_datetime (self, dt);
 
       e_cal_component_set_dtend (self->component, component_dt);
       e_cal_component_commit_sequence (self->component);
 
       g_object_notify (G_OBJECT (self), "date-end");
 
-      e_cal_component_free_datetime (&current);
       e_cal_component_free_datetime (component_dt);
       g_free (component_dt);
     }
@@ -872,29 +1037,21 @@ gcal_event_set_date_start (GcalEvent *self,
 {
   g_return_if_fail (GCAL_IS_EVENT (self));
 
-  if (self->dt_start != dt &&
-      (!self->dt_start || !dt ||
-       (self->dt_start && dt && !g_date_time_equal (self->dt_start, dt))))
+  if (self->dt_start != dt)
     {
-      ECalComponentDateTime *component_dt, current;
+      ECalComponentDateTime *component_dt;
 
       g_clear_pointer (&self->dt_start, g_date_time_unref);
       self->dt_start = g_date_time_ref (dt);
 
-      /* Retrieve the current timezone */
-      e_cal_component_get_dtstart (self->component, &current);
-
       /* Setup the ECalComponent's datetime value */
-      component_dt = build_component_from_datetime (self,
-                                                    icaltimezone_get_builtin_timezone_from_tzid (current.tzid),
-                                                    dt);
+      component_dt = build_component_from_datetime (self, dt);
 
       e_cal_component_set_dtstart (self->component, component_dt);
       e_cal_component_commit_sequence (self->component);
 
       g_object_notify (G_OBJECT (self), "date-start");
 
-      e_cal_component_free_datetime (&current);
       e_cal_component_free_datetime (component_dt);
       g_free (component_dt);
     }
@@ -950,6 +1107,22 @@ gcal_event_set_description (GcalEvent   *self,
     }
 }
 
+/**
+ * gcal_event_has_recurrence:
+ * @self: a #GcalEvent
+ *
+ * Returns whether the event has recurrences or not.
+ *
+ * Returns: %TRUE if @self has recurrences, %FALSE otherwise
+ */
+gboolean
+gcal_event_has_recurrence (GcalEvent *self)
+{
+  g_return_val_if_fail (GCAL_IS_EVENT (self), FALSE);
+
+  return e_cal_component_has_recurrences (self->component);
+}
+
 /**
  * gcal_event_has_alarms:
  * @self: a #GcalEvent
@@ -1334,7 +1507,7 @@ gcal_event_set_timezone (GcalEvent *self,
  * @self: a #GcalEvent
  *
  * Retrieves the unique identifier of the event. The UID
- * consists of '<source id>:<event id>:<recurrency id>'.
+ * consists of '[source id]:[event id]:[recurrency id]'.
  * If there's no recurrency assigned, the last field is
  * ignored.
  *
@@ -1502,3 +1675,62 @@ gcal_event_compare_with_current (GcalEvent *event1,
 
   return 0;
 }
+
+/**
+ * gcal_event_set_recurrence:
+ * @self: a #GcalEvent
+ * @recur: (nullable): a #GcalRecurrence
+ *
+ * Sets the recurrence struct of the event to @recur
+ * and adds the corresponding rrule to the event.
+ */
+void
+gcal_event_set_recurrence (GcalEvent      *self,
+                           GcalRecurrence *recur)
+{
+  ECalComponent *comp;
+  icalcomponent *icalcomp;
+  icalproperty *prop;
+  struct icalrecurrencetype *rrule;
+
+  g_return_if_fail (GCAL_IS_EVENT (self));
+
+  rrule = gcal_recurrence_to_rrule (recur);
+
+  comp = gcal_event_get_component (self);
+  icalcomp = e_cal_component_get_icalcomponent (comp);
+
+  g_clear_pointer (&self->recurrence, gcal_recurrence_unref);
+  self->recurrence = gcal_recurrence_copy (recur);
+
+  prop = icalcomponent_get_first_property (icalcomp, ICAL_RRULE_PROPERTY);
+
+  if (prop)
+    {
+      icalproperty_set_rrule (prop, *rrule);
+    }
+  else
+    {
+      prop = icalproperty_new_rrule (*rrule);
+      icalcomponent_add_property (icalcomp, prop);
+    }
+
+  /* Rescan the component for the changes to be detected and registered */
+  e_cal_component_rescan (comp);
+}
+
+/**
+ * gcal_event_get_recurrence:
+ * @self: a #GcalEvent
+ *
+ * Gets the recurrence struct @recur of the event.
+ *
+ * Returns: (transfer full): a #GcalRecurrence
+ */
+GcalRecurrence*
+gcal_event_get_recurrence (GcalEvent *self)
+{
+  g_return_val_if_fail (GCAL_IS_EVENT (self), NULL);
+
+  return self->recurrence;
+}
