@@ -16,6 +16,8 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#define G_LOG_DOMAIN "GcalQuickAddPopover"
+
 #include <glib/gi18n.h>
 
 #include "gcal-quick-add-popover.h"
@@ -39,6 +41,8 @@ struct _GcalQuickAddPopover
 
   GtkWidget          *selected_row;
 
+  gboolean            clock_format_24h;
+
   GcalManager        *manager;
 };
 
@@ -69,8 +73,8 @@ create_row_for_source (GcalManager *manager,
                        ESource     *source,
                        gboolean     writable)
 {
+  cairo_surface_t *surface;
   GtkWidget *row, *box, *icon, *label, *selected_icon;
-  GdkPixbuf *circle_pixbuf;
   GdkRGBA color;
   gchar *tooltip, *parent_name;
 
@@ -83,8 +87,8 @@ create_row_for_source (GcalManager *manager,
 
   /* The icon with the source color */
   get_color_name_from_source (source, &color);
-  circle_pixbuf = get_circle_pixbuf_from_color (&color, 16);
-  icon = gtk_image_new_from_pixbuf (circle_pixbuf);
+  surface = get_circle_surface_from_color (&color, 16);
+  icon = gtk_image_new_from_surface (surface);
 
   gtk_container_add (GTK_CONTAINER (box), icon);
 
@@ -131,7 +135,7 @@ create_row_for_source (GcalManager *manager,
   gtk_widget_show (box);
   gtk_widget_show (row);
 
-  g_clear_object (&circle_pixbuf);
+  g_clear_pointer (&surface, cairo_surface_destroy);
   g_free (parent_name);
   g_free (tooltip);
 
@@ -172,8 +176,8 @@ static void
 select_row (GcalQuickAddPopover *self,
             GtkListBoxRow       *row)
 {
+  cairo_surface_t *surface;
   GtkWidget *icon;
-  GdkPixbuf *circle_pixbuf;
   ESource *source;
   GdkRGBA color;
 
@@ -196,68 +200,282 @@ select_row (GcalQuickAddPopover *self,
   gtk_label_set_label (GTK_LABEL (self->calendar_name_label), e_source_get_display_name (source));
 
   get_color_name_from_source (source, &color);
-  circle_pixbuf = get_circle_pixbuf_from_color (&color, 16);
-  gtk_image_set_from_pixbuf (GTK_IMAGE (self->color_image), circle_pixbuf);
+  surface = get_circle_surface_from_color (&color, 16);
+  gtk_image_set_from_surface (GTK_IMAGE (self->color_image), surface);
 
   /* Return to the events page */
   gtk_stack_set_visible_child_name (GTK_STACK (self->stack), "events");
 
   /* Focus back the event Entry */
-  gtk_entry_grab_focus_without_selecting (GTK_ENTRY (self->summary_entry));
+  if (gtk_widget_get_visible (GTK_WIDGET (self)))
+    gtk_entry_grab_focus_without_selecting (GTK_ENTRY (self->summary_entry));
 
-  g_clear_object (&circle_pixbuf);
+  g_clear_pointer (&surface, cairo_surface_destroy);
+}
+
+static gint
+get_number_of_days_from_today (GDateTime *day)
+{
+  GDate today, event_day;
+  gint n_days;
+
+  g_date_set_dmy (&today,
+                  g_date_time_get_day_of_month (g_date_time_new_now_local ()),
+                  g_date_time_get_month (g_date_time_new_now_local ()),
+                  g_date_time_get_year (g_date_time_new_now_local ()));
+
+  g_date_set_dmy (&event_day,
+                  g_date_time_get_day_of_month (day),
+                  g_date_time_get_month (day),
+                  g_date_time_get_year (day));
+  n_days = g_date_days_between (&event_day, &today);
+
+  return n_days;
+}
+
+static gchar*
+get_date_string_for_multiday (GDateTime *start,
+                              GDateTime *end)
+{
+  gint n_days;
+  gchar *start_date_str, *end_date_str, *date_string;
+
+  gchar *start_date_weekdays_strings [] = {
+    N_("from next Monday"),
+    N_("from next Tuesday"),
+    N_("from next Wednesday"),
+    N_("from next Thursday"),
+    N_("from next Friday"),
+    N_("from next Saturday"),
+    N_("from next Sunday"),
+    NULL
+  };
+
+  gchar *end_date_weekdays_strings [] = {
+    N_("to next Monday"),
+    N_("to next Tuesday"),
+    N_("to next Wednesday"),
+    N_("to next Thursday"),
+    N_("to next Friday"),
+    N_("to next Saturday"),
+    N_("to next Sunday"),
+    NULL
+  };
+
+  gchar *month_names [] = {
+    N_("January"),
+    N_("February"),
+    N_("March"),
+    N_("April"),
+    N_("May"),
+    N_("June"),
+    N_("July"),
+    N_("August"),
+    N_("September"),
+    N_("October"),
+    N_("November"),
+    N_("December"),
+    NULL
+  };
+
+  start_date_str = NULL;
+  n_days = get_number_of_days_from_today (start);
+
+  if (n_days == 0)
+    {
+      start_date_str = g_strdup_printf (_("from Today"));
+    }
+  else if (n_days == -1)
+    {
+      start_date_str = g_strdup_printf (_("from Tomorrow"));
+    }
+  else if (n_days == 1)
+    {
+      start_date_str = g_strdup_printf (_("from Yesterday"));
+    }
+  else if (n_days < -1 && n_days > -8)
+    {
+      gchar *start_weekday_str;
+
+      start_weekday_str = start_date_weekdays_strings [g_date_time_get_day_of_week (start) - 1];
+      start_date_str = g_strdup_printf ("%s", start_weekday_str);
+    }
+  else
+    {
+      gchar *day_number_str;
+
+      day_number_str = g_strdup_printf ("%u", g_date_time_get_day_of_month (start));
+      /* Translators:
+       * this is the format string for representing a date consisting of a month
+       * name and a date of month.
+       */
+      start_date_str = g_strdup_printf (_("from %1$s %2$s"),
+                                        month_names [g_date_time_get_month (start) - 1],
+                                        day_number_str);
+      g_free (day_number_str);
+    }
+
+  end_date_str = NULL;
+  n_days = get_number_of_days_from_today (end);
+
+  if (n_days == 0)
+    {
+      end_date_str = g_strdup_printf (_("to Today"));
+    }
+  else if (n_days == -1)
+    {
+      end_date_str = g_strdup_printf (_("to Tomorrow"));
+    }
+  else if (n_days == 1)
+    {
+      end_date_str = g_strdup_printf (_("to Yesterday"));
+    }
+  else if (n_days < -1 && n_days > -8)
+    {
+      gchar *end_weekday_str;
+
+      end_weekday_str = end_date_weekdays_strings [g_date_time_get_day_of_week (end) - 1];
+      end_date_str = g_strdup_printf ("%s", end_weekday_str);
+    }
+  else
+    {
+      gchar *day_number_str;
+
+      day_number_str = g_strdup_printf ("%u", g_date_time_get_day_of_month (end));
+      /* Translators:
+       * this is the format string for representing a date consisting of a month
+       * name and a date of month.
+       */
+      end_date_str = g_strdup_printf (_("to %1$s %2$s"),
+                                      month_names [g_date_time_get_month (end) - 1],
+                                      day_number_str);
+      g_free (day_number_str);
+    }
+
+  /* Translators: %1$s is the start date (e.g. "from Today") and %2$s is the end date (e.g. "to Tomorrow") */
+  date_string = g_strdup_printf (_("New Event %1$s %2$s"),
+                                 start_date_str,
+                                 end_date_str);
+  return date_string;
+}
+
+static gchar*
+get_date_string_for_day (GDateTime *day)
+{
+  gchar *string_for_date;
+  gint n_days;
+
+  string_for_date = NULL;
+  n_days = get_number_of_days_from_today (day);
+
+  if (n_days == 0)
+    {
+      string_for_date = g_strdup_printf (_("New Event Today"));
+    }
+  else if (n_days == -1)
+    {
+      string_for_date = g_strdup_printf (_("New Event Tomorrow"));
+    }
+  else if (n_days == 1)
+    {
+      string_for_date = g_strdup_printf (_("New Event Yesterday"));
+    }
+  else if (n_days < -1 && n_days > -8)
+    {
+       gchar *event_weekday;
+       gchar *event_weekday_names [] = {
+         N_("New Event next Monday"),
+         N_("New Event next Tuesday"),
+         N_("New Event next Wednesday"),
+         N_("New Event next Thursday"),
+         N_("New Event next Friday"),
+         N_("New Event next Saturday"),
+         N_("New Event next Sunday"),
+         NULL
+       };
+
+       event_weekday = gettext (event_weekday_names [g_date_time_get_day_of_week (day) - 1]);
+       string_for_date = g_strdup_printf ("%s", event_weekday);
+    }
+  else
+    {
+      const gchar *event_month;
+      gchar *event_month_names [] = {
+        /* Translators: %d is the numeric day of month */
+        N_("New Event on January %d"),
+        N_("New Event on February %d"),
+        N_("New Event on March %d"),
+        N_("New Event on April %d"),
+        N_("New Event on May %d"),
+        N_("New Event on June %d"),
+        N_("New Event on July %d"),
+        N_("New Event on August %d"),
+        N_("New Event on September %d"),
+        N_("New Event on October %d"),
+        N_("New Event on November %d"),
+        N_("New Event on December %d"),
+        NULL
+      };
+
+      event_month = gettext (event_month_names [g_date_time_get_month (day) - 1]);
+      string_for_date = g_strdup_printf (event_month, g_date_time_get_day_of_month (day));
+    }
+
+  return string_for_date;
 }
 
 static void
 update_header (GcalQuickAddPopover *self)
 {
-  icaltimetype *dtstart;
-  struct tm stm_date, etm_date;
-  gboolean all_day;
-  gchar start[64], end[64];
+  gboolean multiday_or_timed;
   gchar *title_date;
 
-  all_day = datetime_is_date (self->date_start) && (self->date_end ? datetime_is_date (self->date_end) : TRUE);
-  dtstart = datetime_to_icaltime (self->date_start);
+  multiday_or_timed = self->date_end &&
+                      (g_date_time_difference (self->date_end, self->date_start) / G_TIME_SPAN_DAY > 1 ||
+                       g_date_time_difference (self->date_end, self->date_start) / G_TIME_SPAN_MINUTE > 1);
 
-  /* Translators:
-   * this is the format string for representing a date consisting of a month name
-   * and a date of month.
-   */
-  stm_date = icaltimetype_to_tm (dtstart);
-  e_utf8_strftime_fix_am_pm (start, 64, C_("event date format", "%B %d"), &stm_date);
-
-  if (self->date_end &&
-      (!all_day ||
-       (all_day && g_date_time_difference (self->date_end, self->date_start) / G_TIME_SPAN_DAY > 1)))
+  if (multiday_or_timed)
     {
-      icaltimetype *dtend;
+      gboolean all_day;
 
-      dtend = datetime_to_icaltime (self->date_end);
+      all_day = datetime_is_date (self->date_start) && (self->date_end ? datetime_is_date (self->date_end) : TRUE);
 
-      if (dtend->is_date)
-        icaltime_adjust (dtend, -1, 0, 0, 0);
+      if (all_day &&
+          (g_date_time_difference (self->date_end, self->date_start) / G_TIME_SPAN_DAY > 1 &&
+           g_date_time_difference (self->date_end, self->date_start) / G_TIME_SPAN_MINUTE > 1))
+        {
+          title_date = get_date_string_for_multiday (self->date_start, g_date_time_add_days (self->date_end, -1));
+        }
+      else
+        {
+          g_autofree gchar *start_hour, *end_hour, *event_date_name;
+          gchar *hour_format;
 
-      /* Translators:
-       * this is the format string for representing a date consisting of a month name
-       * and a date of month.
-       */
-      etm_date = icaltimetype_to_tm (dtend);
-      e_utf8_strftime_fix_am_pm (end, 64, C_("event date format", "%B %d"), &etm_date);
+          if (self->clock_format_24h)
+              hour_format = "%R";
+          else
+              hour_format = "%I:%M %P";
 
-      title_date = g_strdup_printf (_("New Event from %s to %s"), start, end);
+          start_hour = g_date_time_format (self->date_start, hour_format);
+          end_hour = g_date_time_format (self->date_end, hour_format);
 
-      g_free (dtend);
+          event_date_name = get_date_string_for_day (self->date_start);
+          /* Translators: %1$s is the event name, %2$s is the start hour, and %3$s is the end hour */
+          title_date = g_strdup_printf (_("%1$s, %2$s – %3$s"),
+                                        event_date_name,
+                                        start_hour,
+                                        end_hour);
+        }
     }
   else
     {
-      title_date = g_strdup_printf (_("New Event on %s"), start);
+      g_autofree gchar *event_date_name;
+      event_date_name = get_date_string_for_day (self->date_start);
+      title_date = g_strdup_printf ("%s", event_date_name);
     }
 
   gtk_label_set_label (GTK_LABEL (self->title_label), title_date);
-
   g_free (title_date);
-  g_free (dtstart);
 }
 
 static void
@@ -309,8 +527,8 @@ on_source_changed (GcalManager         *manager,
                    ESource             *source,
                    GcalQuickAddPopover *self)
 {
+  cairo_surface_t *surface;
   GtkWidget *row, *color_icon, *name_label;
-  GdkPixbuf *circle_pixbuf;
   GdkRGBA color;
 
   row = get_row_for_source (self, source);
@@ -325,7 +543,7 @@ on_source_changed (GcalManager         *manager,
     {
       on_source_added (self->manager,
                        source,
-                       gcal_manager_source_enabled (self->manager, source),
+                       is_source_enabled (source),
                        self);
 
       row = get_row_for_source (self, source);
@@ -339,10 +557,10 @@ on_source_changed (GcalManager         *manager,
 
   /* Setup the source color, in case it changed */
   get_color_name_from_source (source, &color);
-  circle_pixbuf = get_circle_pixbuf_from_color (&color, 16);
-  gtk_image_set_from_pixbuf (GTK_IMAGE (color_icon), circle_pixbuf);
+  surface = get_circle_surface_from_color (&color, 16);
+  gtk_image_set_from_surface (GTK_IMAGE (color_icon), surface);
 
-  g_clear_object (&circle_pixbuf);
+  g_clear_pointer (&surface, cairo_surface_destroy);
 
   /* Also setup the row name, in case we just changed the source name */
   gtk_label_set_text (GTK_LABEL (name_label), e_source_get_display_name (source));
@@ -413,27 +631,21 @@ edit_or_create_event (GcalQuickAddPopover *self,
                       GtkWidget           *button)
 {
   ECalComponent *component;
-  GDateTime *date_start, *date_end, *now;
+  GDateTime *date_start, *date_end;
   GTimeZone *tz;
   GcalEvent *event;
   ESource *source;
   const gchar *summary;
-  gboolean all_day, single_day, is_today;
+  gboolean all_day, single_day;
 
   if (!self->selected_row)
     return;
 
-  now = g_date_time_new_now_local ();
   source = g_object_get_data (G_OBJECT (self->selected_row), "source");
 
-  /*
-   * We consider all day events multiday and/or non-today events.
-   * Events on today starts now and lasts 1 hour.
-   */
   single_day = datetime_compare_date (self->date_end, self->date_start) == 0;
-  is_today = single_day && datetime_compare_date (now, self->date_start) == 0;
-  all_day = datetime_compare_date (self->date_end, self->date_start) > 1 || !is_today;
-
+  all_day = datetime_compare_date (self->date_end, self->date_start) > 1 ||
+            g_date_time_compare (self->date_end, self->date_start) == 0;
   tz = all_day ? g_time_zone_new_utc () : g_time_zone_new_local ();
 
   /* Gather start date */
@@ -441,8 +653,8 @@ edit_or_create_event (GcalQuickAddPopover *self,
                                 g_date_time_get_year (self->date_start),
                                 g_date_time_get_month (self->date_start),
                                 g_date_time_get_day_of_month (self->date_start),
-                                all_day ? 0 : g_date_time_get_hour (now),
-                                all_day ? 0 : g_date_time_get_minute (now),
+                                g_date_time_get_hour (self->date_start),
+                                g_date_time_get_minute (self->date_start),
                                 0);
 
   /* Gather date end */
@@ -452,8 +664,8 @@ edit_or_create_event (GcalQuickAddPopover *self,
                                   g_date_time_get_year (self->date_end),
                                   g_date_time_get_month (self->date_end),
                                   g_date_time_get_day_of_month (self->date_end) + (single_day && all_day ? 1 : 0),
-                                  all_day ? 0 : g_date_time_get_hour (now) + (is_today ? 1 : 0),
-                                  all_day ? 0 : g_date_time_get_minute (now),
+                                  g_date_time_get_hour (self->date_end),
+                                  g_date_time_get_minute (self->date_end),
                                   0);
     }
   else
@@ -565,7 +777,28 @@ gcal_quick_add_popover_set_property (GObject      *object,
       break;
 
     case PROP_MANAGER:
-      gcal_quick_add_popover_set_manager (self, g_value_get_object (value));
+      if (g_set_object (&self->manager, g_value_get_object (value)))
+        {
+          GcalManager *manager;
+          GList *sources, *l;
+
+          /* Add currently leaded sources */
+          manager = self->manager;
+          sources = gcal_manager_get_sources_connected (manager);
+
+          for (l = sources; l != NULL; l = g_list_next (l))
+            on_source_added (manager, l->data, gcal_manager_is_client_writable (manager, l->data), self);
+
+          g_list_free (sources);
+
+          /* Connect to the manager signals and keep the list updates */
+          g_signal_connect (manager, "source-added", G_CALLBACK (on_source_added), self);
+          g_signal_connect (manager, "source-changed", G_CALLBACK (on_source_changed), self);
+          g_signal_connect (manager, "source-removed", G_CALLBACK (on_source_removed), self);
+          g_signal_connect_swapped (manager, "notify::default-calendar", G_CALLBACK (update_default_calendar_row), self);
+
+          g_object_notify (G_OBJECT (self), "manager");
+        }
       break;
 
     default:
@@ -681,6 +914,8 @@ gcal_quick_add_popover_init (GcalQuickAddPopover *self)
   gtk_widget_init_template (GTK_WIDGET (self));
 
   gtk_list_box_set_sort_func (GTK_LIST_BOX (self->calendars_listbox), sort_func, NULL, NULL);
+
+  self->clock_format_24h = is_clock_format_24h ();
 }
 
 GtkWidget*
@@ -689,41 +924,6 @@ gcal_quick_add_popover_new (void)
   return g_object_new (GCAL_TYPE_QUICK_ADD_POPOVER, NULL);
 }
 
-/**
- * gcal_quick_add_popover_set_manager:
- * @self: a #GcalQuickAddPopover
- * @manager: a #GcalManager
- *
- * Sets the manager of the popover.
- */
-void
-gcal_quick_add_popover_set_manager (GcalQuickAddPopover *self,
-                                    GcalManager         *manager)
-{
-  g_return_if_fail (GCAL_IS_QUICK_ADD_POPOVER (self));
-
-  if (g_set_object (&self->manager, manager))
-    {
-      GList *sources, *l;
-
-      /* Add currently leaded sources */
-      sources = gcal_manager_get_sources_connected (manager);
-
-      for (l = sources; l != NULL; l = g_list_next (l))
-        on_source_added (manager, l->data, gcal_manager_is_client_writable (manager, l->data), self);
-
-      g_list_free (sources);
-
-      /* Connect to the manager signals and keep the list updates */
-      g_signal_connect (manager, "source-added", G_CALLBACK (on_source_added), self);
-      g_signal_connect (manager, "source-changed", G_CALLBACK (on_source_changed), self);
-      g_signal_connect (manager, "source-removed", G_CALLBACK (on_source_removed), self);
-      g_signal_connect_swapped (manager, "notify::default-calendar", G_CALLBACK (update_default_calendar_row), self);
-
-      g_object_notify (G_OBJECT (self), "manager");
-    }
-}
-
 /**
  * gcal_quick_add_popover_get_date_start:
  * @self: a #GcalQuickAddPopover
